from tkinter import Widget
from typing import Union

from .tkinter_widgets import *

AnyWidget = Union[
    Button,
    Label,
    Entry,
    Canvas,
    Frame,
    LabelFrame,
    Checkbutton,
    Menu,
    Listbox,
    Menubutton,
    Message,
    OptionMenu,
    Text,
    Scrollbar,
    Spinbox,
    TtkButton,
    TtkLabel,
    TtkEntry,
    Combobox,
    TtkFrame,
    TtkLabelFrame,
    TtkCheckbutton,
    TtkMenubutton,
    TtkOptionMenu,
    Notebook,
    Panedwindow,
    Progressbar,
    Radiobutton,
    Scale,
    LabeledScale,
    TtkScrollbar,
    Spinbox,
    Separator,
    Sizegrip,
    Widget
]